# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, List, Mapping

from lightspark.requests.requester import Requester

from .NodeAddress import NodeAddress
from .NodeAddress import from_json as NodeAddress_from_json


@dataclass
class NodeToAddressesConnection:
    """A connection between a node and the addresses it has announced for itself on Lightning Network."""

    requester: Requester

    count: int
    """The total count of objects in this connection, using the current filters. It is different from the number of objects returned in the current page (in the `entities` field)."""

    entities: List[NodeAddress]
    """The addresses for the current page of this connection."""


FRAGMENT = """
fragment NodeToAddressesConnectionFragment on NodeToAddressesConnection {
    __typename
    node_to_addresses_connection_count: count
    node_to_addresses_connection_entities: entities {
        __typename
        node_address_address: address
        node_address_type: type
    }
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> NodeToAddressesConnection:
    return NodeToAddressesConnection(
        requester=requester,
        count=obj["node_to_addresses_connection_count"],
        entities=list(
            map(
                lambda e: NodeAddress_from_json(requester, e),
                obj["node_to_addresses_connection_entities"],
            )
        ),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, List, Mapping

from lightspark.requests.requester import Requester

from .Channel import Channel
from .Channel import from_json as Channel_from_json
from .PageInfo import PageInfo
from .PageInfo import from_json as PageInfo_from_json


@dataclass
class LightsparkNodeToChannelsConnection:
    requester: Requester

    page_info: PageInfo
    """An object that holds pagination information about the objects in this connection."""

    count: int
    """The total count of objects in this connection, using the current filters. It is different from the number of objects returned in the current page (in the `entities` field)."""

    entities: List[Channel]
    """The channels for the current page of this connection."""


FRAGMENT = """
fragment LightsparkNodeToChannelsConnectionFragment on LightsparkNodeToChannelsConnection {
    __typename
    lightspark_node_to_channels_connection_page_info: page_info {
        __typename
        page_info_has_next_page: has_next_page
        page_info_has_previous_page: has_previous_page
        page_info_start_cursor: start_cursor
        page_info_end_cursor: end_cursor
    }
    lightspark_node_to_channels_connection_count: count
    lightspark_node_to_channels_connection_entities: entities {
        id
    }
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> LightsparkNodeToChannelsConnection:
    return LightsparkNodeToChannelsConnection(
        requester=requester,
        page_info=PageInfo_from_json(
            requester, obj["lightspark_node_to_channels_connection_page_info"]
        ),
        count=obj["lightspark_node_to_channels_connection_count"],
        entities=list(
            map(
                lambda e: Channel_from_json(requester, e),
                obj["lightspark_node_to_channels_connection_entities"],
            )
        ),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping, Optional


@dataclass
class PayInvoiceInput:
    node_id: str
    """The node from where you want to send the payment."""

    encoded_invoice: str
    """The invoice you want to pay (as defined by the BOLT11 standard)."""

    timeout_secs: int
    """The timeout in seconds that we will try to make the payment."""

    maximum_fees_msats: int
    """The maximum amount of fees that you want to pay for this payment to be sent, expressed in msats."""

    amount_msats: Optional[int]
    """The amount you will pay for this invoice, expressed in msats. It should ONLY be set when the invoice amount is zero."""


def from_json(obj: Mapping[str, Any]) -> PayInvoiceInput:
    return PayInvoiceInput(
        node_id=obj["pay_invoice_input_node_id"],
        encoded_invoice=obj["pay_invoice_input_encoded_invoice"],
        timeout_secs=obj["pay_invoice_input_timeout_secs"],
        maximum_fees_msats=obj["pay_invoice_input_maximum_fees_msats"],
        amount_msats=obj["pay_invoice_input_amount_msats"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Mapping, Optional

from lightspark.exceptions import LightsparkException
from lightspark.objects.BitcoinNetwork import BitcoinNetwork
from lightspark.objects.LightsparkNodePurpose import LightsparkNodePurpose
from lightspark.objects.LightsparkNodeStatus import LightsparkNodeStatus
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum, parse_enum_optional

from .BitcoinNetwork import BitcoinNetwork
from .BlockchainBalance import from_json as BlockchainBalance_from_json
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Entity import Entity
from .NodeAddressType import NodeAddressType
from .NodeToAddressesConnection import NodeToAddressesConnection
from .NodeToAddressesConnection import from_json as NodeToAddressesConnection_from_json
from .Secret import from_json as Secret_from_json


@dataclass
class Node(Entity):
    """This interface represents a lightning node that can be connected to the Lightning Network to send and receive transactions."""

    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    alias: Optional[str]
    """A name that identifies the node. It has no importance in terms of operating the node, it is just a way to identify and search for commercial services or popular nodes. This alias can be changed at any time by the node operator."""

    bitcoin_network: BitcoinNetwork
    """The Bitcoin Network this node is deployed in."""

    color: Optional[str]
    """A hexadecimal string that describes a color. For example "#000000" is black, "#FFFFFF" is white. It has no importance in terms of operating the node, it is just a way to visually differentiate nodes. That color can be changed at any time by the node operator."""

    conductivity: Optional[int]
    """A summary metric used to capture how well positioned a node is to send, receive, or route transactions efficiently. Maximizing a node's conductivity helps a node’s transactions to be capital efficient. The value is an integer ranging between 0 and 10 (bounds included)."""

    display_name: str
    """The name of this node in the network. It will be the most human-readable option possible, depending on the data available for this node."""

    public_key: Optional[str]
    """The public key of this node. It acts as a unique identifier of this node in the Lightning Network."""
    typename: str

    def get_addresses(
        self, first: Optional[int] = None, types: Optional[List[NodeAddressType]] = None
    ) -> NodeToAddressesConnection:
        json = self.requester.execute_graphql(
            """
query FetchNodeToAddressesConnection($entity_id: ID!, $first: Int, $types: [NodeAddressType!]) {
    entity(id: $entity_id) {
        ... on Node {
            addresses(, first: $first, types: $types) {
                __typename
                node_to_addresses_connection_count: count
                node_to_addresses_connection_entities: entities {
                    __typename
                    node_address_address: address
                    node_address_type: type
                }
            }
        }
    }
}
            """,
            {"entity_id": self.id, "first": first, "types": types},
        )
        connection = json["entity"]["addresses"]
        return NodeToAddressesConnection_from_json(self.requester, connection)


FRAGMENT = """
fragment NodeFragment on Node {
    __typename
    ... on GraphNode {
        __typename
        graph_node_id: id
        graph_node_created_at: created_at
        graph_node_updated_at: updated_at
        graph_node_alias: alias
        graph_node_bitcoin_network: bitcoin_network
        graph_node_color: color
        graph_node_conductivity: conductivity
        graph_node_display_name: display_name
        graph_node_public_key: public_key
    }
    ... on LightsparkNode {
        __typename
        lightspark_node_id: id
        lightspark_node_created_at: created_at
        lightspark_node_updated_at: updated_at
        lightspark_node_alias: alias
        lightspark_node_bitcoin_network: bitcoin_network
        lightspark_node_color: color
        lightspark_node_conductivity: conductivity
        lightspark_node_display_name: display_name
        lightspark_node_public_key: public_key
        lightspark_node_account: account {
            id
        }
        lightspark_node_blockchain_balance: blockchain_balance {
            __typename
            blockchain_balance_total_balance: total_balance {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
            blockchain_balance_confirmed_balance: confirmed_balance {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
            blockchain_balance_unconfirmed_balance: unconfirmed_balance {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
            blockchain_balance_locked_balance: locked_balance {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
            blockchain_balance_required_reserve: required_reserve {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
            blockchain_balance_available_balance: available_balance {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
        }
        lightspark_node_encrypted_signing_private_key: encrypted_signing_private_key {
            __typename
            secret_encrypted_value: encrypted_value
            secret_cipher: cipher
        }
        lightspark_node_total_balance: total_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        lightspark_node_total_local_balance: total_local_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        lightspark_node_local_balance: local_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        lightspark_node_purpose: purpose
        lightspark_node_remote_balance: remote_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        lightspark_node_status: status
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> Node:
    if obj["__typename"] == "GraphNode":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.GraphNode import GraphNode

        return GraphNode(
            requester=requester,
            typename="GraphNode",
            id=obj["graph_node_id"],
            created_at=obj["graph_node_created_at"],
            updated_at=obj["graph_node_updated_at"],
            alias=obj["graph_node_alias"],
            bitcoin_network=parse_enum(
                BitcoinNetwork, obj["graph_node_bitcoin_network"]
            ),
            color=obj["graph_node_color"],
            conductivity=obj["graph_node_conductivity"],
            display_name=obj["graph_node_display_name"],
            public_key=obj["graph_node_public_key"],
        )
    if obj["__typename"] == "LightsparkNode":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.LightsparkNode import LightsparkNode

        return LightsparkNode(
            requester=requester,
            typename="LightsparkNode",
            id=obj["lightspark_node_id"],
            created_at=obj["lightspark_node_created_at"],
            updated_at=obj["lightspark_node_updated_at"],
            alias=obj["lightspark_node_alias"],
            bitcoin_network=parse_enum(
                BitcoinNetwork, obj["lightspark_node_bitcoin_network"]
            ),
            color=obj["lightspark_node_color"],
            conductivity=obj["lightspark_node_conductivity"],
            display_name=obj["lightspark_node_display_name"],
            public_key=obj["lightspark_node_public_key"],
            account_id=obj["lightspark_node_account"]["id"],
            blockchain_balance=BlockchainBalance_from_json(
                requester, obj["lightspark_node_blockchain_balance"]
            )
            if obj["lightspark_node_blockchain_balance"]
            else None,
            encrypted_signing_private_key=Secret_from_json(
                requester, obj["lightspark_node_encrypted_signing_private_key"]
            )
            if obj["lightspark_node_encrypted_signing_private_key"]
            else None,
            total_balance=CurrencyAmount_from_json(
                requester, obj["lightspark_node_total_balance"]
            )
            if obj["lightspark_node_total_balance"]
            else None,
            total_local_balance=CurrencyAmount_from_json(
                requester, obj["lightspark_node_total_local_balance"]
            )
            if obj["lightspark_node_total_local_balance"]
            else None,
            local_balance=CurrencyAmount_from_json(
                requester, obj["lightspark_node_local_balance"]
            )
            if obj["lightspark_node_local_balance"]
            else None,
            purpose=parse_enum_optional(
                LightsparkNodePurpose, obj["lightspark_node_purpose"]
            ),
            remote_balance=CurrencyAmount_from_json(
                requester, obj["lightspark_node_remote_balance"]
            )
            if obj["lightspark_node_remote_balance"]
            else None,
            status=parse_enum_optional(
                LightsparkNodeStatus, obj["lightspark_node_status"]
            ),
        )
    graphql_typename = obj["__typename"]
    raise LightsparkException(
        "UNKNOWN_INTERFACE",
        f"Couldn't find a concrete type for interface Node corresponding to the typename={graphql_typename}",
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, List, Mapping

from lightspark.requests.requester import Requester

from .Hop import Hop
from .Hop import from_json as Hop_from_json


@dataclass
class OutgoingPaymentAttemptToHopsConnection:
    """The connection from an outgoing payment attempt to the list of sequential hops that define the path from sender node to recipient node."""

    requester: Requester

    count: int
    """The total count of objects in this connection, using the current filters. It is different from the number of objects returned in the current page (in the `entities` field)."""

    entities: List[Hop]
    """The hops for the current page of this connection."""


FRAGMENT = """
fragment OutgoingPaymentAttemptToHopsConnectionFragment on OutgoingPaymentAttemptToHopsConnection {
    __typename
    outgoing_payment_attempt_to_hops_connection_count: count
    outgoing_payment_attempt_to_hops_connection_entities: entities {
        id
    }
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> OutgoingPaymentAttemptToHopsConnection:
    return OutgoingPaymentAttemptToHopsConnection(
        requester=requester,
        count=obj["outgoing_payment_attempt_to_hops_connection_count"],
        entities=list(
            map(
                lambda e: Hop_from_json(requester, e),
                obj["outgoing_payment_attempt_to_hops_connection_entities"],
            )
        ),
    )



from enum import Enum
from typing import List, Optional, TypeVar, Type

T = TypeVar("T", bound=Enum)


def parse_enum(enum_type: Type[T], value: str) -> T:
    try:
        return enum_type[value]
    except KeyError:
        return enum_type["___FUTURE_VALUE___"]


def parse_enum_list(enum_type: Type[T], values: List[str]) -> List[T]:
    return list(map(lambda value: parse_enum(enum_type, value), values))


def parse_enum_optional(enum_type: Type[T], value: Optional[str]) -> Optional[T]:
    return parse_enum(enum_type, value) if value else None


def parse_optional_list_of_enums(
    enum_type: Type[T], values: Optional[List[str]]
) -> Optional[List[T]]:
    return parse_enum_list(enum_type, values) if values else None


def parse_list_of_optional_enums(
    enum_type: Type[T], values: List[Optional[str]]
) -> List[Optional[T]]:
    return list(map(lambda value: parse_enum_optional(enum_type, value), values))


def parse_optional_list_of_optional_enums(
    enum_type: Type[T], values: Optional[List[Optional[str]]]
) -> Optional[List[Optional[T]]]:
    return parse_list_of_optional_enums(enum_type, values) if values else None



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Mapping

from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum_list

from .Entity import Entity
from .Permission import Permission


@dataclass
class ApiToken(Entity):
    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    client_id: str
    """An opaque identifier that should be used as a client_id (or username) in the HTTP Basic Authentication scheme when issuing requests against the Lightspark API."""

    name: str
    """An arbitrary name chosen by the creator of the token to help identify the token in the list of tokens that have been created for the account."""

    permissions: List[Permission]
    """A list of permissions granted to the token."""
    typename: str


FRAGMENT = """
fragment ApiTokenFragment on ApiToken {
    __typename
    api_token_id: id
    api_token_created_at: created_at
    api_token_updated_at: updated_at
    api_token_client_id: client_id
    api_token_name: name
    api_token_permissions: permissions
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> ApiToken:
    return ApiToken(
        requester=requester,
        typename="ApiToken",
        id=obj["api_token_id"],
        created_at=obj["api_token_created_at"],
        updated_at=obj["api_token_updated_at"],
        client_id=obj["api_token_client_id"],
        name=obj["api_token_name"],
        permissions=parse_enum_list(Permission, obj["api_token_permissions"]),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.exceptions import LightsparkException
from lightspark.objects.CurrencyUnit import CurrencyUnit
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum

from .CurrencyUnit import CurrencyUnit


@dataclass
class CurrencyAmount:
    """Represents the value and unit for an amount of currency."""

    requester: Requester

    original_value: int
    """The original numeric value for this CurrencyAmount."""

    original_unit: CurrencyUnit
    """The original unit of currency for this CurrencyAmount."""

    preferred_currency_unit: CurrencyUnit
    """The unit of user's preferred currency."""

    preferred_currency_value_rounded: int
    """The rounded numeric value for this CurrencyAmount in the very base level of user's preferred currency. For example, for USD, the value will be in cents."""

    preferred_currency_value_approx: float
    """The approximate float value for this CurrencyAmount in the very base level of user's preferred currency. For example, for USD, the value will be in cents."""

    _CONVERSION_MAP = {
        CurrencyUnit.BITCOIN: {
            CurrencyUnit.BITCOIN: lambda v: v,
            CurrencyUnit.MICROBITCOIN: lambda v: v * 1_000_000,
            CurrencyUnit.MILLIBITCOIN: lambda v: v * 1000,
            CurrencyUnit.MILLISATOSHI: lambda v: v * 100_000_000_000,
            CurrencyUnit.NANOBITCOIN: lambda v: v * 1_000_000_000,
            CurrencyUnit.SATOSHI: lambda v: v * 100_000_000,
        },
        CurrencyUnit.MICROBITCOIN: {
            CurrencyUnit.BITCOIN: lambda v: round(v / 1_000_000),
            CurrencyUnit.MICROBITCOIN: lambda v: v,
            CurrencyUnit.MILLIBITCOIN: lambda v: round(v / 1000),
            CurrencyUnit.MILLISATOSHI: lambda v: v * 100_000,
            CurrencyUnit.NANOBITCOIN: lambda v: v * 1000,
            CurrencyUnit.SATOSHI: lambda v: v * 100,
        },
        CurrencyUnit.MILLIBITCOIN: {
            CurrencyUnit.BITCOIN: lambda v: round(v / 1_000),
            CurrencyUnit.MICROBITCOIN: lambda v: v * 1000,
            CurrencyUnit.MILLIBITCOIN: lambda v: v,
            CurrencyUnit.MILLISATOSHI: lambda v: v * 100_000_000,
            CurrencyUnit.NANOBITCOIN: lambda v: v * 1_000_000,
            CurrencyUnit.SATOSHI: lambda v: v * 100_000,
        },
        CurrencyUnit.MILLISATOSHI: {
            CurrencyUnit.BITCOIN: lambda v: round(v / 100_000_000_000),
            CurrencyUnit.MICROBITCOIN: lambda v: round(v / 100_000),
            CurrencyUnit.MILLIBITCOIN: lambda v: round(v / 100_000_000),
            CurrencyUnit.MILLISATOSHI: lambda v: v,
            CurrencyUnit.NANOBITCOIN: lambda v: round(v / 100),
            CurrencyUnit.SATOSHI: lambda v: round(v / 1000),
        },
        CurrencyUnit.NANOBITCOIN: {
            CurrencyUnit.BITCOIN: lambda v: round(v / 1_000_000_000),
            CurrencyUnit.MICROBITCOIN: lambda v: round(v / 1000),
            CurrencyUnit.MILLIBITCOIN: lambda v: round(v / 1_000_000),
            CurrencyUnit.MILLISATOSHI: lambda v: v * 100,
            CurrencyUnit.NANOBITCOIN: lambda v: v,
            CurrencyUnit.SATOSHI: lambda v: round(v / 10),
        },
        CurrencyUnit.SATOSHI: {
            CurrencyUnit.BITCOIN: lambda v: round(v / 100_000_000),
            CurrencyUnit.MICROBITCOIN: lambda v: round(v / 100),
            CurrencyUnit.MILLIBITCOIN: lambda v: round(v / 100_000),
            CurrencyUnit.MILLISATOSHI: lambda v: v * 1000,
            CurrencyUnit.NANOBITCOIN: lambda v: v * 10,
            CurrencyUnit.SATOSHI: lambda v: v,
        },
    }

    def convert_to(self, unit: CurrencyUnit) -> "CurrencyAmount":
        try:
            conversion_fn = self._CONVERSION_MAP[self.original_unit][unit]
        except KeyError as e:
            raise LightsparkException(
                "CONVERSION_ERROR",
                f"Cannot convert from {self.original_unit} to {unit}",
            ) from e
        return CurrencyAmount(
            requester=self.requester,
            original_unit=self.original_unit,
            original_value=self.original_value,
            preferred_currency_value_rounded=conversion_fn(self.original_value),
            preferred_currency_value_approx=conversion_fn(self.original_value),
            preferred_currency_unit=unit,
        )


FRAGMENT = """
fragment CurrencyAmountFragment on CurrencyAmount {
    __typename
    currency_amount_original_value: original_value
    currency_amount_original_unit: original_unit
    currency_amount_preferred_currency_unit: preferred_currency_unit
    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> CurrencyAmount:
    return CurrencyAmount(
        requester=requester,
        original_value=obj["currency_amount_original_value"],
        original_unit=parse_enum(CurrencyUnit, obj["currency_amount_original_unit"]),
        preferred_currency_unit=parse_enum(
            CurrencyUnit, obj["currency_amount_preferred_currency_unit"]
        ),
        preferred_currency_value_rounded=obj[
            "currency_amount_preferred_currency_value_rounded"
        ],
        preferred_currency_value_approx=obj[
            "currency_amount_preferred_currency_value_approx"
        ],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping


@dataclass
class DeleteApiTokenInput:
    api_token_id: str


def from_json(obj: Mapping[str, Any]) -> DeleteApiTokenInput:
    return DeleteApiTokenInput(
        api_token_id=obj["delete_api_token_input_api_token_id"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester


@dataclass
class RequestWithdrawalOutput:
    requester: Requester

    request_id: str
    """The request that is created for this withdrawal."""


FRAGMENT = """
fragment RequestWithdrawalOutputFragment on RequestWithdrawalOutput {
    __typename
    request_withdrawal_output_request: request {
        id
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> RequestWithdrawalOutput:
    return RequestWithdrawalOutput(
        requester=requester,
        request_id=obj["request_withdrawal_output_request"]["id"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from enum import Enum


class IncomingPaymentAttemptStatus(Enum):
    """Enum that enumerates all the possible status of an incoming payment attempt."""

    ___FUTURE_VALUE___ = "___FUTURE_VALUE___"
    """This is an enum value that represents future values that could be added in the future. Clients should support unknown values as more of them could be added without notice."""
    ACCEPTED = "ACCEPTED"
    SETTLED = "SETTLED"
    CANCELED = "CANCELED"
    UNKNOWN = "UNKNOWN"



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping, Optional

from lightspark.requests.requester import Requester

from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json


@dataclass
class ChannelToTransactionsConnection:
    requester: Requester

    count: int
    """The total count of objects in this connection, using the current filters. It is different from the number of objects returned in the current page (in the `entities` field)."""

    average_fee: Optional[CurrencyAmount]
    """The average fee for the transactions that transited through this channel, according to the filters and constraints of the connection."""

    total_amount_transacted: Optional[CurrencyAmount]
    """The total amount transacted for the transactions that transited through this channel, according to the filters and constraints of the connection."""

    total_fees: Optional[CurrencyAmount]
    """The total amount of fees for the transactions that transited through this channel, according to the filters and constraints of the connection."""


FRAGMENT = """
fragment ChannelToTransactionsConnectionFragment on ChannelToTransactionsConnection {
    __typename
    channel_to_transactions_connection_count: count
    channel_to_transactions_connection_average_fee: average_fee {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_to_transactions_connection_total_amount_transacted: total_amount_transacted {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_to_transactions_connection_total_fees: total_fees {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> ChannelToTransactionsConnection:
    return ChannelToTransactionsConnection(
        requester=requester,
        count=obj["channel_to_transactions_connection_count"],
        average_fee=CurrencyAmount_from_json(
            requester, obj["channel_to_transactions_connection_average_fee"]
        )
        if obj["channel_to_transactions_connection_average_fee"]
        else None,
        total_amount_transacted=CurrencyAmount_from_json(
            requester, obj["channel_to_transactions_connection_total_amount_transacted"]
        )
        if obj["channel_to_transactions_connection_total_amount_transacted"]
        else None,
        total_fees=CurrencyAmount_from_json(
            requester, obj["channel_to_transactions_connection_total_fees"]
        )
        if obj["channel_to_transactions_connection_total_fees"]
        else None,
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, Mapping, Optional

from lightspark.requests.requester import Requester

from .Balances import Balances
from .Balances import from_json as Balances_from_json
from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Entity import Entity


@dataclass
class Wallet(Entity):
    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    last_login_at: Optional[datetime]
    """The date and time when the wallet user last logged in."""

    balances: Optional[Balances]
    """The balances that describe the funds in this wallet."""

    third_party_identifier: str
    """The unique identifier of this wallet, as provided by the Lightspark Customer during login."""
    typename: str

    def get_total_amount_received(
        self,
        created_after_date: Optional[datetime] = None,
        created_before_date: Optional[datetime] = None,
    ) -> CurrencyAmount:
        json = self.requester.execute_graphql(
            """
query FetchWalletTotalAmountReceived($entity_id: ID!, $created_after_date: DateTime, $created_before_date: DateTime) {
    entity(id: $entity_id) {
        ... on Wallet {
            total_amount_received(, created_after_date: $created_after_date, created_before_date: $created_before_date) {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
        }
    }
}
            """,
            {
                "entity_id": self.id,
                "created_after_date": created_after_date,
                "created_before_date": created_before_date,
            },
        )
        connection = json["entity"]["total_amount_received"]
        return CurrencyAmount_from_json(self.requester, connection)

    def get_total_amount_sent(
        self,
        created_after_date: Optional[datetime] = None,
        created_before_date: Optional[datetime] = None,
    ) -> CurrencyAmount:
        json = self.requester.execute_graphql(
            """
query FetchWalletTotalAmountSent($entity_id: ID!, $created_after_date: DateTime, $created_before_date: DateTime) {
    entity(id: $entity_id) {
        ... on Wallet {
            total_amount_sent(, created_after_date: $created_after_date, created_before_date: $created_before_date) {
                __typename
                currency_amount_original_value: original_value
                currency_amount_original_unit: original_unit
                currency_amount_preferred_currency_unit: preferred_currency_unit
                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
            }
        }
    }
}
            """,
            {
                "entity_id": self.id,
                "created_after_date": created_after_date,
                "created_before_date": created_before_date,
            },
        )
        connection = json["entity"]["total_amount_sent"]
        return CurrencyAmount_from_json(self.requester, connection)


FRAGMENT = """
fragment WalletFragment on Wallet {
    __typename
    wallet_id: id
    wallet_created_at: created_at
    wallet_updated_at: updated_at
    wallet_last_login_at: last_login_at
    wallet_balances: balances {
        __typename
        balances_owned_balance: owned_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        balances_available_to_send_balance: available_to_send_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        balances_available_to_withdraw_balance: available_to_withdraw_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
    }
    wallet_third_party_identifier: third_party_identifier
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> Wallet:
    return Wallet(
        requester=requester,
        typename="Wallet",
        id=obj["wallet_id"],
        created_at=obj["wallet_created_at"],
        updated_at=obj["wallet_updated_at"],
        last_login_at=obj["wallet_last_login_at"],
        balances=Balances_from_json(requester, obj["wallet_balances"])
        if obj["wallet_balances"]
        else None,
        third_party_identifier=obj["wallet_third_party_identifier"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Mapping, Optional

from lightspark.objects.BitcoinNetwork import BitcoinNetwork
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum

from .BitcoinNetwork import BitcoinNetwork
from .Entity import Entity
from .Node import Node
from .NodeAddressType import NodeAddressType
from .NodeToAddressesConnection import NodeToAddressesConnection
from .NodeToAddressesConnection import from_json as NodeToAddressesConnection_from_json


@dataclass
class GraphNode(Node, Entity):
    """This is a node on the Lightning Network, managed by a third party. The information about this node is public data that has been obtained by observing the Lightning Network."""

    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    alias: Optional[str]
    """A name that identifies the node. It has no importance in terms of operating the node, it is just a way to identify and search for commercial services or popular nodes. This alias can be changed at any time by the node operator."""

    bitcoin_network: BitcoinNetwork
    """The Bitcoin Network this node is deployed in."""

    color: Optional[str]
    """A hexadecimal string that describes a color. For example "#000000" is black, "#FFFFFF" is white. It has no importance in terms of operating the node, it is just a way to visually differentiate nodes. That color can be changed at any time by the node operator."""

    conductivity: Optional[int]
    """A summary metric used to capture how well positioned a node is to send, receive, or route transactions efficiently. Maximizing a node's conductivity helps a node’s transactions to be capital efficient. The value is an integer ranging between 0 and 10 (bounds included)."""

    display_name: str
    """The name of this node in the network. It will be the most human-readable option possible, depending on the data available for this node."""

    public_key: Optional[str]
    """The public key of this node. It acts as a unique identifier of this node in the Lightning Network."""
    typename: str

    def get_addresses(
        self, first: Optional[int] = None, types: Optional[List[NodeAddressType]] = None
    ) -> NodeToAddressesConnection:
        json = self.requester.execute_graphql(
            """
query FetchNodeToAddressesConnection($entity_id: ID!, $first: Int, $types: [NodeAddressType!]) {
    entity(id: $entity_id) {
        ... on GraphNode {
            addresses(, first: $first, types: $types) {
                __typename
                node_to_addresses_connection_count: count
                node_to_addresses_connection_entities: entities {
                    __typename
                    node_address_address: address
                    node_address_type: type
                }
            }
        }
    }
}
            """,
            {"entity_id": self.id, "first": first, "types": types},
        )
        connection = json["entity"]["addresses"]
        return NodeToAddressesConnection_from_json(self.requester, connection)


FRAGMENT = """
fragment GraphNodeFragment on GraphNode {
    __typename
    graph_node_id: id
    graph_node_created_at: created_at
    graph_node_updated_at: updated_at
    graph_node_alias: alias
    graph_node_bitcoin_network: bitcoin_network
    graph_node_color: color
    graph_node_conductivity: conductivity
    graph_node_display_name: display_name
    graph_node_public_key: public_key
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> GraphNode:
    return GraphNode(
        requester=requester,
        typename="GraphNode",
        id=obj["graph_node_id"],
        created_at=obj["graph_node_created_at"],
        updated_at=obj["graph_node_updated_at"],
        alias=obj["graph_node_alias"],
        bitcoin_network=parse_enum(BitcoinNetwork, obj["graph_node_bitcoin_network"]),
        color=obj["graph_node_color"],
        conductivity=obj["graph_node_conductivity"],
        display_name=obj["graph_node_display_name"],
        public_key=obj["graph_node_public_key"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Mapping, Optional

from lightspark.objects.ChannelStatus import ChannelStatus
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum_optional

from .ChannelFees import ChannelFees
from .ChannelFees import from_json as ChannelFees_from_json
from .ChannelStatus import ChannelStatus
from .ChannelToTransactionsConnection import ChannelToTransactionsConnection
from .ChannelToTransactionsConnection import (
    from_json as ChannelToTransactionsConnection_from_json,
)
from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Entity import Entity
from .TransactionType import TransactionType


@dataclass
class Channel(Entity):
    """An object that represents a payment channel between two nodes in the Lightning Network."""

    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    funding_transaction_id: Optional[str]
    """The transaction that funded the channel upon channel opening."""

    capacity: Optional[CurrencyAmount]
    """The total amount of funds in this channel, including the channel balance on the local node, the channel balance on the remote node and the on-chain fees to close the channel."""

    local_balance: Optional[CurrencyAmount]
    """The channel balance on the local node."""

    local_unsettled_balance: Optional[CurrencyAmount]
    """The channel balance on the local node that is currently allocated to in-progress payments."""

    remote_balance: Optional[CurrencyAmount]
    """The channel balance on the remote node."""

    remote_unsettled_balance: Optional[CurrencyAmount]
    """The channel balance on the remote node that is currently allocated to in-progress payments."""

    unsettled_balance: Optional[CurrencyAmount]
    """The channel balance that is currently allocated to in-progress payments."""

    total_balance: Optional[CurrencyAmount]
    """The total balance in this channel, including the channel balance on both local and remote nodes."""

    status: Optional[ChannelStatus]
    """The current status of this channel."""

    estimated_force_closure_wait_minutes: Optional[int]
    """The estimated time to wait for the channel's hash timelock contract to expire when force closing the channel. It is in unit of minutes."""

    commit_fee: Optional[CurrencyAmount]
    """The amount to be paid in fees for the current set of commitment transactions."""

    fees: Optional[ChannelFees]
    """The fees charged for routing payments through this channel."""

    remote_node_id: Optional[str]
    """If known, the remote node of the channel."""

    local_node_id: str
    """The local Lightspark node of the channel."""

    short_channel_id: Optional[str]
    """The unique identifier of the channel on Lightning Network, which is the location in the chain that the channel was confirmed. The format is <block-height>:<tx-index>:<tx-output>."""
    typename: str

    def get_uptime_percentage(
        self,
        after_date: Optional[datetime] = None,
        before_date: Optional[datetime] = None,
    ) -> Optional[int]:
        json = self.requester.execute_graphql(
            """
query FetchChannelUptimePercentage($entity_id: ID!, $after_date: DateTime, $before_date: DateTime) {
    entity(id: $entity_id) {
        ... on Channel {
            uptime_percentage(, after_date: $after_date, before_date: $before_date)
        }
    }
}
            """,
            {
                "entity_id": self.id,
                "after_date": after_date,
                "before_date": before_date,
            },
        )
        connection = json["entity"]["uptime_percentage"]
        return connection

    def get_transactions(
        self,
        types: Optional[List[TransactionType]] = None,
        after_date: Optional[datetime] = None,
        before_date: Optional[datetime] = None,
    ) -> ChannelToTransactionsConnection:
        json = self.requester.execute_graphql(
            """
query FetchChannelToTransactionsConnection($entity_id: ID!, $types: [TransactionType!], $after_date: DateTime, $before_date: DateTime) {
    entity(id: $entity_id) {
        ... on Channel {
            transactions(, types: $types, after_date: $after_date, before_date: $before_date) {
                __typename
                channel_to_transactions_connection_count: count
                channel_to_transactions_connection_average_fee: average_fee {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                channel_to_transactions_connection_total_amount_transacted: total_amount_transacted {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                channel_to_transactions_connection_total_fees: total_fees {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
            }
        }
    }
}
            """,
            {
                "entity_id": self.id,
                "types": types,
                "after_date": after_date,
                "before_date": before_date,
            },
        )
        connection = json["entity"]["transactions"]
        return ChannelToTransactionsConnection_from_json(self.requester, connection)


FRAGMENT = """
fragment ChannelFragment on Channel {
    __typename
    channel_id: id
    channel_created_at: created_at
    channel_updated_at: updated_at
    channel_funding_transaction: funding_transaction {
        id
    }
    channel_capacity: capacity {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_local_balance: local_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_local_unsettled_balance: local_unsettled_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_remote_balance: remote_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_remote_unsettled_balance: remote_unsettled_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_unsettled_balance: unsettled_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_total_balance: total_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_status: status
    channel_estimated_force_closure_wait_minutes: estimated_force_closure_wait_minutes
    channel_commit_fee: commit_fee {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_fees: fees {
        __typename
        channel_fees_base_fee: base_fee {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        channel_fees_fee_rate_per_mil: fee_rate_per_mil
    }
    channel_remote_node: remote_node {
        id
    }
    channel_local_node: local_node {
        id
    }
    channel_short_channel_id: short_channel_id
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> Channel:
    return Channel(
        requester=requester,
        typename="Channel",
        id=obj["channel_id"],
        created_at=obj["channel_created_at"],
        updated_at=obj["channel_updated_at"],
        funding_transaction_id=obj["channel_funding_transaction"]["id"]
        if obj["channel_funding_transaction"]
        else None,
        capacity=CurrencyAmount_from_json(requester, obj["channel_capacity"])
        if obj["channel_capacity"]
        else None,
        local_balance=CurrencyAmount_from_json(requester, obj["channel_local_balance"])
        if obj["channel_local_balance"]
        else None,
        local_unsettled_balance=CurrencyAmount_from_json(
            requester, obj["channel_local_unsettled_balance"]
        )
        if obj["channel_local_unsettled_balance"]
        else None,
        remote_balance=CurrencyAmount_from_json(
            requester, obj["channel_remote_balance"]
        )
        if obj["channel_remote_balance"]
        else None,
        remote_unsettled_balance=CurrencyAmount_from_json(
            requester, obj["channel_remote_unsettled_balance"]
        )
        if obj["channel_remote_unsettled_balance"]
        else None,
        unsettled_balance=CurrencyAmount_from_json(
            requester, obj["channel_unsettled_balance"]
        )
        if obj["channel_unsettled_balance"]
        else None,
        total_balance=CurrencyAmount_from_json(requester, obj["channel_total_balance"])
        if obj["channel_total_balance"]
        else None,
        status=parse_enum_optional(ChannelStatus, obj["channel_status"]),
        estimated_force_closure_wait_minutes=obj[
            "channel_estimated_force_closure_wait_minutes"
        ],
        commit_fee=CurrencyAmount_from_json(requester, obj["channel_commit_fee"])
        if obj["channel_commit_fee"]
        else None,
        fees=ChannelFees_from_json(requester, obj["channel_fees"])
        if obj["channel_fees"]
        else None,
        remote_node_id=obj["channel_remote_node"]["id"]
        if obj["channel_remote_node"]
        else None,
        local_node_id=obj["channel_local_node"]["id"],
        short_channel_id=obj["channel_short_channel_id"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, List, Mapping, Optional

from lightspark.utils.enums import parse_optional_list_of_enums

from .PaymentFailureReason import PaymentFailureReason
from .RoutingTransactionFailureReason import RoutingTransactionFailureReason


@dataclass
class TransactionFailures:
    payment_failures: Optional[List[PaymentFailureReason]]

    routing_transaction_failures: Optional[List[RoutingTransactionFailureReason]]


def from_json(obj: Mapping[str, Any]) -> TransactionFailures:
    return TransactionFailures(
        payment_failures=parse_optional_list_of_enums(
            PaymentFailureReason, obj["transaction_failures_payment_failures"]
        ),
        routing_transaction_failures=parse_optional_list_of_enums(
            RoutingTransactionFailureReason,
            obj["transaction_failures_routing_transaction_failures"],
        ),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Mapping, Optional

from lightspark.objects.BitcoinNetwork import BitcoinNetwork
from lightspark.objects.LightsparkNodePurpose import LightsparkNodePurpose
from lightspark.objects.LightsparkNodeStatus import LightsparkNodeStatus
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum, parse_enum_optional

from .BitcoinNetwork import BitcoinNetwork
from .BlockchainBalance import BlockchainBalance
from .BlockchainBalance import from_json as BlockchainBalance_from_json
from .ChannelStatus import ChannelStatus
from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Entity import Entity
from .LightsparkNodePurpose import LightsparkNodePurpose
from .LightsparkNodeStatus import LightsparkNodeStatus
from .LightsparkNodeToChannelsConnection import LightsparkNodeToChannelsConnection
from .LightsparkNodeToChannelsConnection import (
    from_json as LightsparkNodeToChannelsConnection_from_json,
)
from .Node import Node
from .NodeAddressType import NodeAddressType
from .NodeToAddressesConnection import NodeToAddressesConnection
from .NodeToAddressesConnection import from_json as NodeToAddressesConnection_from_json
from .Secret import Secret
from .Secret import from_json as Secret_from_json


@dataclass
class LightsparkNode(Node, Entity):
    """This is a node that is managed by Lightspark and is managed within the current connected account. It contains many details about the node configuration, state, and metadata."""

    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when the entity was first created."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    alias: Optional[str]
    """A name that identifies the node. It has no importance in terms of operating the node, it is just a way to identify and search for commercial services or popular nodes. This alias can be changed at any time by the node operator."""

    bitcoin_network: BitcoinNetwork
    """The Bitcoin Network this node is deployed in."""

    color: Optional[str]
    """A hexadecimal string that describes a color. For example "#000000" is black, "#FFFFFF" is white. It has no importance in terms of operating the node, it is just a way to visually differentiate nodes. That color can be changed at any time by the node operator."""

    conductivity: Optional[int]
    """A summary metric used to capture how well positioned a node is to send, receive, or route transactions efficiently. Maximizing a node's conductivity helps a node’s transactions to be capital efficient. The value is an integer ranging between 0 and 10 (bounds included)."""

    display_name: str
    """The name of this node in the network. It will be the most human-readable option possible, depending on the data available for this node."""

    public_key: Optional[str]
    """The public key of this node. It acts as a unique identifier of this node in the Lightning Network."""

    account_id: str
    """The account that owns this LightsparkNode."""

    blockchain_balance: Optional[BlockchainBalance]
    """The details of the balance of this node on the Bitcoin Network."""

    encrypted_signing_private_key: Optional[Secret]
    """The private key client is using to sign a GraphQL request which will be verified at LND."""

    total_balance: Optional[CurrencyAmount]
    """The sum of the balance on the Bitcoin Network, channel balances, and commit fees on this node."""

    total_local_balance: Optional[CurrencyAmount]
    """The total sum of the channel balances (online and offline) on this node."""

    local_balance: Optional[CurrencyAmount]
    """The sum of the channel balances (online only) that are available to send on this node."""

    purpose: Optional[LightsparkNodePurpose]
    """The main purpose of this node. It is used by Lightspark for optimizations on the node's channels."""

    remote_balance: Optional[CurrencyAmount]
    """The sum of the channel balances that are available to receive on this node."""

    status: Optional[LightsparkNodeStatus]
    """The current status of this node."""
    typename: str

    def get_addresses(
        self, first: Optional[int] = None, types: Optional[List[NodeAddressType]] = None
    ) -> NodeToAddressesConnection:
        json = self.requester.execute_graphql(
            """
query FetchNodeToAddressesConnection($entity_id: ID!, $first: Int, $types: [NodeAddressType!]) {
    entity(id: $entity_id) {
        ... on LightsparkNode {
            addresses(, first: $first, types: $types) {
                __typename
                node_to_addresses_connection_count: count
                node_to_addresses_connection_entities: entities {
                    __typename
                    node_address_address: address
                    node_address_type: type
                }
            }
        }
    }
}
            """,
            {"entity_id": self.id, "first": first, "types": types},
        )
        connection = json["entity"]["addresses"]
        return NodeToAddressesConnection_from_json(self.requester, connection)

    def get_channels(
        self,
        first: Optional[int] = None,
        statuses: Optional[List[ChannelStatus]] = None,
    ) -> LightsparkNodeToChannelsConnection:
        json = self.requester.execute_graphql(
            """
query FetchLightsparkNodeToChannelsConnection($entity_id: ID!, $first: Int, $statuses: [ChannelStatus!]) {
    entity(id: $entity_id) {
        ... on LightsparkNode {
            channels(, first: $first, statuses: $statuses) {
                __typename
                lightspark_node_to_channels_connection_page_info: page_info {
                    __typename
                    page_info_has_next_page: has_next_page
                    page_info_has_previous_page: has_previous_page
                    page_info_start_cursor: start_cursor
                    page_info_end_cursor: end_cursor
                }
                lightspark_node_to_channels_connection_count: count
                lightspark_node_to_channels_connection_entities: entities {
                    __typename
                    channel_id: id
                    channel_created_at: created_at
                    channel_updated_at: updated_at
                    channel_funding_transaction: funding_transaction {
                        id
                    }
                    channel_capacity: capacity {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_local_balance: local_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_local_unsettled_balance: local_unsettled_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_remote_balance: remote_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_remote_unsettled_balance: remote_unsettled_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_unsettled_balance: unsettled_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_total_balance: total_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_status: status
                    channel_estimated_force_closure_wait_minutes: estimated_force_closure_wait_minutes
                    channel_commit_fee: commit_fee {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    channel_fees: fees {
                        __typename
                        channel_fees_base_fee: base_fee {
                            __typename
                            currency_amount_original_value: original_value
                            currency_amount_original_unit: original_unit
                            currency_amount_preferred_currency_unit: preferred_currency_unit
                            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                        }
                        channel_fees_fee_rate_per_mil: fee_rate_per_mil
                    }
                    channel_remote_node: remote_node {
                        id
                    }
                    channel_local_node: local_node {
                        id
                    }
                    channel_short_channel_id: short_channel_id
                }
            }
        }
    }
}
            """,
            {"entity_id": self.id, "first": first, "statuses": statuses},
        )
        connection = json["entity"]["channels"]
        return LightsparkNodeToChannelsConnection_from_json(self.requester, connection)


FRAGMENT = """
fragment LightsparkNodeFragment on LightsparkNode {
    __typename
    lightspark_node_id: id
    lightspark_node_created_at: created_at
    lightspark_node_updated_at: updated_at
    lightspark_node_alias: alias
    lightspark_node_bitcoin_network: bitcoin_network
    lightspark_node_color: color
    lightspark_node_conductivity: conductivity
    lightspark_node_display_name: display_name
    lightspark_node_public_key: public_key
    lightspark_node_account: account {
        id
    }
    lightspark_node_blockchain_balance: blockchain_balance {
        __typename
        blockchain_balance_total_balance: total_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        blockchain_balance_confirmed_balance: confirmed_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        blockchain_balance_unconfirmed_balance: unconfirmed_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        blockchain_balance_locked_balance: locked_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        blockchain_balance_required_reserve: required_reserve {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        blockchain_balance_available_balance: available_balance {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
    }
    lightspark_node_encrypted_signing_private_key: encrypted_signing_private_key {
        __typename
        secret_encrypted_value: encrypted_value
        secret_cipher: cipher
    }
    lightspark_node_total_balance: total_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    lightspark_node_total_local_balance: total_local_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    lightspark_node_local_balance: local_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    lightspark_node_purpose: purpose
    lightspark_node_remote_balance: remote_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    lightspark_node_status: status
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> LightsparkNode:
    return LightsparkNode(
        requester=requester,
        typename="LightsparkNode",
        id=obj["lightspark_node_id"],
        created_at=obj["lightspark_node_created_at"],
        updated_at=obj["lightspark_node_updated_at"],
        alias=obj["lightspark_node_alias"],
        bitcoin_network=parse_enum(
            BitcoinNetwork, obj["lightspark_node_bitcoin_network"]
        ),
        color=obj["lightspark_node_color"],
        conductivity=obj["lightspark_node_conductivity"],
        display_name=obj["lightspark_node_display_name"],
        public_key=obj["lightspark_node_public_key"],
        account_id=obj["lightspark_node_account"]["id"],
        blockchain_balance=BlockchainBalance_from_json(
            requester, obj["lightspark_node_blockchain_balance"]
        )
        if obj["lightspark_node_blockchain_balance"]
        else None,
        encrypted_signing_private_key=Secret_from_json(
            requester, obj["lightspark_node_encrypted_signing_private_key"]
        )
        if obj["lightspark_node_encrypted_signing_private_key"]
        else None,
        total_balance=CurrencyAmount_from_json(
            requester, obj["lightspark_node_total_balance"]
        )
        if obj["lightspark_node_total_balance"]
        else None,
        total_local_balance=CurrencyAmount_from_json(
            requester, obj["lightspark_node_total_local_balance"]
        )
        if obj["lightspark_node_total_local_balance"]
        else None,
        local_balance=CurrencyAmount_from_json(
            requester, obj["lightspark_node_local_balance"]
        )
        if obj["lightspark_node_local_balance"]
        else None,
        purpose=parse_enum_optional(
            LightsparkNodePurpose, obj["lightspark_node_purpose"]
        ),
        remote_balance=CurrencyAmount_from_json(
            requester, obj["lightspark_node_remote_balance"]
        )
        if obj["lightspark_node_remote_balance"]
        else None,
        status=parse_enum_optional(LightsparkNodeStatus, obj["lightspark_node_status"]),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

import json
import logging
import re
import secrets
from datetime import datetime, timedelta, timezone
from platform import python_version, release, system
from typing import Any, Mapping, Optional
from urllib.parse import urlparse

import requests
from requests.auth import HTTPBasicAuth
from requests.utils import default_user_agent

from lightspark.exceptions import LightsparkException
from lightspark.requests.encoder import Encoder
from lightspark.utils.crypto import sign_payload
from lightspark.version import __version__

DEFAULT_BASE_URL = "https://api.lightspark.com/graphql/server/2023-04-04"

logger = logging.getLogger("lightspark")


class Requester:
    def __init__(
        self,
        api_token_client_id: str,
        api_token_client_secret: str,
        base_url: Optional[str] = None,
        http_host: Optional[str] = None,
    ) -> None:
        self.base_url = base_url or DEFAULT_BASE_URL
        self.graphql_session = requests.Session()
        self.graphql_session.auth = HTTPBasicAuth(
            api_token_client_id, api_token_client_secret
        )

        if http_host:
            self.graphql_session.mount("https://", HTTPSAdapter(http_host))
            self.graphql_session.headers.update({"Host": http_host})

    def execute_graphql(
        self,
        query: str,
        variables: Optional[Mapping[str, Any]],
        signing_key: Optional[bytes] = None,
    ) -> Mapping[str, Any]:
        operation = re.match(r"\s*(?:query|mutation)\s+(\w+)", query, re.IGNORECASE)
        payload = json.dumps(
            {
                "operationName": operation.group(1) if operation else None,
                "query": query,
                "variables": variables or {},
                "nonce": secrets.randbits(32) if signing_key else None,
                "expires_at": (datetime.utcnow() + timedelta(hours=1))
                .replace(tzinfo=timezone.utc)
                .isoformat()
                if signing_key
                else None,
            },
            cls=Encoder,
        ).encode("utf8")

        signing = sign_payload(payload, signing_key) if signing_key else None

        user_agent = self.user_agent_string()
        logger.debug(
            "Sending request to GraphQL with query = %s, payload = %s}", query, payload
        )
        r = self.graphql_session.post(
            url=self.base_url,
            data=payload,
            headers={
                "Content-Type": "application/json",
                "X-GraphQL-Operation": operation.group(1) if operation else None,
                "X-Lightspark-Signing": signing,
                "User-Agent": user_agent + f" {default_user_agent()}",
                "X-Lightspark-SDK": user_agent,
            },
        )
        try:
            r.raise_for_status()
            result = r.json()
            if "errors" in result:
                errors = result["errors"]
                raise LightsparkException(
                    "GRAPHQL_ERROR", f"A GraphQL error occurred: {errors}"
                )  # TODO better error handling
            return result["data"]
        except requests.HTTPError as e:
            logger.error("HTTP request error. Status code: %d", r.status_code)
            raise LightsparkException("HTTP_ERROR", str(e)) from e
        except Exception as e:
            logger.exception(e)

            try:
                logger.error(r.text)
            # TODO pylint is right here... let's make it better.
            except Exception:  # pylint: disable=broad-except
                pass
            raise e

    def user_agent_string(self):
        # Will produce something like: 	lightspark-python-sdk/0.5.1 python/3.11.2 Darwin/22.1.0
        return f"lightspark-python-sdk/{__version__} python/{python_version()} {system()}/{release()}"


class HTTPSAdapter(requests.adapters.HTTPAdapter):
    def __init__(self, server_hostname: str, *args, **kwargs):
        self.server_hostname = server_hostname
        super().__init__(*args, **kwargs)

    def get_connection(self, url, proxies=None):
        url = urlparse(url).geturl()
        return self.poolmanager.connection_from_url(
            url, pool_kwargs={"server_hostname": self.server_hostname}
        )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from typing import Callable, Mapping, Optional, Type, TypeVar

from lightspark.objects.Account import FRAGMENT as AccountFragment
from lightspark.objects.Account import Account
from lightspark.objects.Account import from_json as Account_from_json
from lightspark.objects.ApiToken import FRAGMENT as ApiTokenFragment
from lightspark.objects.ApiToken import ApiToken
from lightspark.objects.ApiToken import from_json as ApiToken_from_json
from lightspark.objects.Channel import FRAGMENT as ChannelFragment
from lightspark.objects.Channel import Channel
from lightspark.objects.Channel import from_json as Channel_from_json
from lightspark.objects.ChannelClosingTransaction import (
    FRAGMENT as ChannelClosingTransactionFragment,
)
from lightspark.objects.ChannelClosingTransaction import ChannelClosingTransaction
from lightspark.objects.ChannelClosingTransaction import (
    from_json as ChannelClosingTransaction_from_json,
)
from lightspark.objects.ChannelOpeningTransaction import (
    FRAGMENT as ChannelOpeningTransactionFragment,
)
from lightspark.objects.ChannelOpeningTransaction import ChannelOpeningTransaction
from lightspark.objects.ChannelOpeningTransaction import (
    from_json as ChannelOpeningTransaction_from_json,
)
from lightspark.objects.Deposit import FRAGMENT as DepositFragment
from lightspark.objects.Deposit import Deposit
from lightspark.objects.Deposit import from_json as Deposit_from_json
from lightspark.objects.Entity import FRAGMENT as EntityFragment
from lightspark.objects.Entity import Entity
from lightspark.objects.GraphNode import FRAGMENT as GraphNodeFragment
from lightspark.objects.GraphNode import GraphNode
from lightspark.objects.GraphNode import from_json as GraphNode_from_json
from lightspark.objects.Hop import FRAGMENT as HopFragment
from lightspark.objects.Hop import Hop
from lightspark.objects.Hop import from_json as Hop_from_json
from lightspark.objects.IncomingPayment import FRAGMENT as IncomingPaymentFragment
from lightspark.objects.IncomingPayment import IncomingPayment
from lightspark.objects.IncomingPayment import from_json as IncomingPayment_from_json
from lightspark.objects.IncomingPaymentAttempt import (
    FRAGMENT as IncomingPaymentAttemptFragment,
)
from lightspark.objects.IncomingPaymentAttempt import IncomingPaymentAttempt
from lightspark.objects.IncomingPaymentAttempt import (
    from_json as IncomingPaymentAttempt_from_json,
)
from lightspark.objects.Invoice import FRAGMENT as InvoiceFragment
from lightspark.objects.Invoice import Invoice
from lightspark.objects.Invoice import from_json as Invoice_from_json
from lightspark.objects.LightningTransaction import (
    FRAGMENT as LightningTransactionFragment,
)
from lightspark.objects.LightningTransaction import LightningTransaction
from lightspark.objects.LightningTransaction import (
    from_json as LightningTransaction_from_json,
)
from lightspark.objects.LightsparkNode import FRAGMENT as LightsparkNodeFragment
from lightspark.objects.LightsparkNode import LightsparkNode
from lightspark.objects.LightsparkNode import from_json as LightsparkNode_from_json
from lightspark.objects.Node import FRAGMENT as NodeFragment
from lightspark.objects.Node import Node
from lightspark.objects.Node import from_json as Node_from_json
from lightspark.objects.OnChainTransaction import FRAGMENT as OnChainTransactionFragment
from lightspark.objects.OnChainTransaction import OnChainTransaction
from lightspark.objects.OnChainTransaction import (
    from_json as OnChainTransaction_from_json,
)
from lightspark.objects.OutgoingPayment import FRAGMENT as OutgoingPaymentFragment
from lightspark.objects.OutgoingPayment import OutgoingPayment
from lightspark.objects.OutgoingPayment import from_json as OutgoingPayment_from_json
from lightspark.objects.OutgoingPaymentAttempt import (
    FRAGMENT as OutgoingPaymentAttemptFragment,
)
from lightspark.objects.OutgoingPaymentAttempt import OutgoingPaymentAttempt
from lightspark.objects.OutgoingPaymentAttempt import (
    from_json as OutgoingPaymentAttempt_from_json,
)
from lightspark.objects.PaymentRequest import FRAGMENT as PaymentRequestFragment
from lightspark.objects.PaymentRequest import PaymentRequest
from lightspark.objects.PaymentRequest import from_json as PaymentRequest_from_json
from lightspark.objects.RoutingTransaction import FRAGMENT as RoutingTransactionFragment
from lightspark.objects.RoutingTransaction import RoutingTransaction
from lightspark.objects.RoutingTransaction import (
    from_json as RoutingTransaction_from_json,
)
from lightspark.objects.Transaction import FRAGMENT as TransactionFragment
from lightspark.objects.Transaction import Transaction
from lightspark.objects.Transaction import from_json as Transaction_from_json
from lightspark.objects.Wallet import FRAGMENT as WalletFragment
from lightspark.objects.Wallet import Wallet
from lightspark.objects.Wallet import from_json as Wallet_from_json
from lightspark.objects.Withdrawal import FRAGMENT as WithdrawalFragment
from lightspark.objects.Withdrawal import Withdrawal
from lightspark.objects.Withdrawal import from_json as Withdrawal_from_json
from lightspark.objects.WithdrawalRequest import FRAGMENT as WithdrawalRequestFragment
from lightspark.objects.WithdrawalRequest import WithdrawalRequest
from lightspark.objects.WithdrawalRequest import (
    from_json as WithdrawalRequest_from_json,
)
from lightspark.requests.requester import Requester

ENTITY = TypeVar("ENTITY", bound=Entity)

ALL_QUERIES: Mapping[Type, str] = {
    Account: """        ... on Account {
            ...AccountFragment
        }
""",
    ApiToken: """        ... on ApiToken {
            ...ApiTokenFragment
        }
""",
    Channel: """        ... on Channel {
            ...ChannelFragment
        }
""",
    ChannelClosingTransaction: """        ... on ChannelClosingTransaction {
            ...ChannelClosingTransactionFragment
        }
""",
    ChannelOpeningTransaction: """        ... on ChannelOpeningTransaction {
            ...ChannelOpeningTransactionFragment
        }
""",
    Deposit: """        ... on Deposit {
            ...DepositFragment
        }
""",
    Entity: """        ... on Entity {
            ...EntityFragment
        }
""",
    GraphNode: """        ... on GraphNode {
            ...GraphNodeFragment
        }
""",
    Hop: """        ... on Hop {
            ...HopFragment
        }
""",
    IncomingPayment: """        ... on IncomingPayment {
            ...IncomingPaymentFragment
        }
""",
    IncomingPaymentAttempt: """        ... on IncomingPaymentAttempt {
            ...IncomingPaymentAttemptFragment
        }
""",
    Invoice: """        ... on Invoice {
            ...InvoiceFragment
        }
""",
    LightningTransaction: """        ... on LightningTransaction {
            ...LightningTransactionFragment
        }
""",
    LightsparkNode: """        ... on LightsparkNode {
            ...LightsparkNodeFragment
        }
""",
    Node: """        ... on Node {
            ...NodeFragment
        }
""",
    OnChainTransaction: """        ... on OnChainTransaction {
            ...OnChainTransactionFragment
        }
""",
    OutgoingPayment: """        ... on OutgoingPayment {
            ...OutgoingPaymentFragment
        }
""",
    OutgoingPaymentAttempt: """        ... on OutgoingPaymentAttempt {
            ...OutgoingPaymentAttemptFragment
        }
""",
    PaymentRequest: """        ... on PaymentRequest {
            ...PaymentRequestFragment
        }
""",
    RoutingTransaction: """        ... on RoutingTransaction {
            ...RoutingTransactionFragment
        }
""",
    Transaction: """        ... on Transaction {
            ...TransactionFragment
        }
""",
    Wallet: """        ... on Wallet {
            ...WalletFragment
        }
""",
    Withdrawal: """        ... on Withdrawal {
            ...WithdrawalFragment
        }
""",
    WithdrawalRequest: """        ... on WithdrawalRequest {
            ...WithdrawalRequestFragment
        }
""",
}
ALL_FRAGMENTS: Mapping[Type, str] = {
    Account: AccountFragment,
    ApiToken: ApiTokenFragment,
    Channel: ChannelFragment,
    ChannelClosingTransaction: ChannelClosingTransactionFragment,
    ChannelOpeningTransaction: ChannelOpeningTransactionFragment,
    Deposit: DepositFragment,
    Entity: EntityFragment,
    GraphNode: GraphNodeFragment,
    Hop: HopFragment,
    IncomingPayment: IncomingPaymentFragment,
    IncomingPaymentAttempt: IncomingPaymentAttemptFragment,
    Invoice: InvoiceFragment,
    LightningTransaction: LightningTransactionFragment,
    LightsparkNode: LightsparkNodeFragment,
    Node: NodeFragment,
    OnChainTransaction: OnChainTransactionFragment,
    OutgoingPayment: OutgoingPaymentFragment,
    OutgoingPaymentAttempt: OutgoingPaymentAttemptFragment,
    PaymentRequest: PaymentRequestFragment,
    RoutingTransaction: RoutingTransactionFragment,
    Transaction: TransactionFragment,
    Wallet: WalletFragment,
    Withdrawal: WithdrawalFragment,
    WithdrawalRequest: WithdrawalRequestFragment,
}
ALL_JSON_LOADERS: Mapping[Type, Callable] = {
    Account: Account_from_json,
    ApiToken: ApiToken_from_json,
    Channel: Channel_from_json,
    ChannelClosingTransaction: ChannelClosingTransaction_from_json,
    ChannelOpeningTransaction: ChannelOpeningTransaction_from_json,
    Deposit: Deposit_from_json,
    GraphNode: GraphNode_from_json,
    Hop: Hop_from_json,
    IncomingPayment: IncomingPayment_from_json,
    IncomingPaymentAttempt: IncomingPaymentAttempt_from_json,
    Invoice: Invoice_from_json,
    LightningTransaction: LightningTransaction_from_json,
    LightsparkNode: LightsparkNode_from_json,
    Node: Node_from_json,
    OnChainTransaction: OnChainTransaction_from_json,
    OutgoingPayment: OutgoingPayment_from_json,
    OutgoingPaymentAttempt: OutgoingPaymentAttempt_from_json,
    PaymentRequest: PaymentRequest_from_json,
    RoutingTransaction: RoutingTransaction_from_json,
    Transaction: Transaction_from_json,
    Wallet: Wallet_from_json,
    Withdrawal: Withdrawal_from_json,
    WithdrawalRequest: WithdrawalRequest_from_json,
}


def get_entity(
    requester: Requester, entity_id: str, entity_class: Type[ENTITY]
) -> Optional[ENTITY]:
    json = requester.execute_graphql(
        f"""
query GetEntity($id: ID!) {{
    entity(id: $id) {{
{ALL_QUERIES[entity_class]}
    }}
}}

{ALL_FRAGMENTS[entity_class]}
""",
        {"id": entity_id},
    )
    if not json["entity"]:
        return None
    return ALL_JSON_LOADERS[entity_class](requester, json["entity"])



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from datetime import datetime
from typing import Any, Mapping, Optional

from lightspark.exceptions import LightsparkException
from lightspark.objects.PaymentFailureReason import PaymentFailureReason
from lightspark.objects.RoutingTransactionFailureReason import (
    RoutingTransactionFailureReason,
)
from lightspark.objects.TransactionStatus import TransactionStatus
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum, parse_enum_optional

from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Entity import Entity
from .PaymentRequestData import from_json as PaymentRequestData_from_json
from .RichText import from_json as RichText_from_json
from .TransactionStatus import TransactionStatus


@dataclass
class Transaction(Entity):
    requester: Requester

    id: str
    """The unique identifier of this entity across all Lightspark systems. Should be treated as an opaque string."""

    created_at: datetime
    """The date and time when this transaction was initiated."""

    updated_at: datetime
    """The date and time when the entity was last updated."""

    status: TransactionStatus
    """The current status of this transaction."""

    resolved_at: Optional[datetime]
    """The date and time when this transaction was completed or failed."""

    amount: CurrencyAmount
    """The amount of money involved in this transaction."""

    transaction_hash: Optional[str]
    """The hash of this transaction, so it can be uniquely identified on the Lightning Network."""
    typename: str


FRAGMENT = """
fragment TransactionFragment on Transaction {
    __typename
    ... on ChannelClosingTransaction {
        __typename
        channel_closing_transaction_id: id
        channel_closing_transaction_created_at: created_at
        channel_closing_transaction_updated_at: updated_at
        channel_closing_transaction_status: status
        channel_closing_transaction_resolved_at: resolved_at
        channel_closing_transaction_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        channel_closing_transaction_transaction_hash: transaction_hash
        channel_closing_transaction_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        channel_closing_transaction_block_hash: block_hash
        channel_closing_transaction_block_height: block_height
        channel_closing_transaction_destination_addresses: destination_addresses
        channel_closing_transaction_num_confirmations: num_confirmations
        channel_closing_transaction_channel: channel {
            id
        }
    }
    ... on ChannelOpeningTransaction {
        __typename
        channel_opening_transaction_id: id
        channel_opening_transaction_created_at: created_at
        channel_opening_transaction_updated_at: updated_at
        channel_opening_transaction_status: status
        channel_opening_transaction_resolved_at: resolved_at
        channel_opening_transaction_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        channel_opening_transaction_transaction_hash: transaction_hash
        channel_opening_transaction_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        channel_opening_transaction_block_hash: block_hash
        channel_opening_transaction_block_height: block_height
        channel_opening_transaction_destination_addresses: destination_addresses
        channel_opening_transaction_num_confirmations: num_confirmations
        channel_opening_transaction_channel: channel {
            id
        }
    }
    ... on Deposit {
        __typename
        deposit_id: id
        deposit_created_at: created_at
        deposit_updated_at: updated_at
        deposit_status: status
        deposit_resolved_at: resolved_at
        deposit_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        deposit_transaction_hash: transaction_hash
        deposit_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        deposit_block_hash: block_hash
        deposit_block_height: block_height
        deposit_destination_addresses: destination_addresses
        deposit_num_confirmations: num_confirmations
        deposit_destination: destination {
            id
        }
    }
    ... on IncomingPayment {
        __typename
        incoming_payment_id: id
        incoming_payment_created_at: created_at
        incoming_payment_updated_at: updated_at
        incoming_payment_status: status
        incoming_payment_resolved_at: resolved_at
        incoming_payment_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        incoming_payment_transaction_hash: transaction_hash
        incoming_payment_origin: origin {
            id
        }
        incoming_payment_destination: destination {
            id
        }
        incoming_payment_payment_request: payment_request {
            id
        }
    }
    ... on OutgoingPayment {
        __typename
        outgoing_payment_id: id
        outgoing_payment_created_at: created_at
        outgoing_payment_updated_at: updated_at
        outgoing_payment_status: status
        outgoing_payment_resolved_at: resolved_at
        outgoing_payment_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        outgoing_payment_transaction_hash: transaction_hash
        outgoing_payment_origin: origin {
            id
        }
        outgoing_payment_destination: destination {
            id
        }
        outgoing_payment_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        outgoing_payment_payment_request_data: payment_request_data {
            __typename
            ... on InvoiceData {
                __typename
                invoice_data_encoded_payment_request: encoded_payment_request
                invoice_data_bitcoin_network: bitcoin_network
                invoice_data_payment_hash: payment_hash
                invoice_data_amount: amount {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                invoice_data_created_at: created_at
                invoice_data_expires_at: expires_at
                invoice_data_memo: memo
                invoice_data_destination: destination {
                    __typename
                    ... on GraphNode {
                        __typename
                        graph_node_id: id
                        graph_node_created_at: created_at
                        graph_node_updated_at: updated_at
                        graph_node_alias: alias
                        graph_node_bitcoin_network: bitcoin_network
                        graph_node_color: color
                        graph_node_conductivity: conductivity
                        graph_node_display_name: display_name
                        graph_node_public_key: public_key
                    }
                    ... on LightsparkNode {
                        __typename
                        lightspark_node_id: id
                        lightspark_node_created_at: created_at
                        lightspark_node_updated_at: updated_at
                        lightspark_node_alias: alias
                        lightspark_node_bitcoin_network: bitcoin_network
                        lightspark_node_color: color
                        lightspark_node_conductivity: conductivity
                        lightspark_node_display_name: display_name
                        lightspark_node_public_key: public_key
                        lightspark_node_account: account {
                            id
                        }
                        lightspark_node_blockchain_balance: blockchain_balance {
                            __typename
                            blockchain_balance_total_balance: total_balance {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                            blockchain_balance_confirmed_balance: confirmed_balance {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                            blockchain_balance_unconfirmed_balance: unconfirmed_balance {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                            blockchain_balance_locked_balance: locked_balance {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                            blockchain_balance_required_reserve: required_reserve {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                            blockchain_balance_available_balance: available_balance {
                                __typename
                                currency_amount_original_value: original_value
                                currency_amount_original_unit: original_unit
                                currency_amount_preferred_currency_unit: preferred_currency_unit
                                currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                                currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                            }
                        }
                        lightspark_node_encrypted_signing_private_key: encrypted_signing_private_key {
                            __typename
                            secret_encrypted_value: encrypted_value
                            secret_cipher: cipher
                        }
                        lightspark_node_total_balance: total_balance {
                            __typename
                            currency_amount_original_value: original_value
                            currency_amount_original_unit: original_unit
                            currency_amount_preferred_currency_unit: preferred_currency_unit
                            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                        }
                        lightspark_node_total_local_balance: total_local_balance {
                            __typename
                            currency_amount_original_value: original_value
                            currency_amount_original_unit: original_unit
                            currency_amount_preferred_currency_unit: preferred_currency_unit
                            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                        }
                        lightspark_node_local_balance: local_balance {
                            __typename
                            currency_amount_original_value: original_value
                            currency_amount_original_unit: original_unit
                            currency_amount_preferred_currency_unit: preferred_currency_unit
                            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                        }
                        lightspark_node_purpose: purpose
                        lightspark_node_remote_balance: remote_balance {
                            __typename
                            currency_amount_original_value: original_value
                            currency_amount_original_unit: original_unit
                            currency_amount_preferred_currency_unit: preferred_currency_unit
                            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                        }
                        lightspark_node_status: status
                    }
                }
            }
        }
        outgoing_payment_failure_reason: failure_reason
        outgoing_payment_failure_message: failure_message {
            __typename
            rich_text_text: text
        }
    }
    ... on RoutingTransaction {
        __typename
        routing_transaction_id: id
        routing_transaction_created_at: created_at
        routing_transaction_updated_at: updated_at
        routing_transaction_status: status
        routing_transaction_resolved_at: resolved_at
        routing_transaction_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        routing_transaction_transaction_hash: transaction_hash
        routing_transaction_incoming_channel: incoming_channel {
            id
        }
        routing_transaction_outgoing_channel: outgoing_channel {
            id
        }
        routing_transaction_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        routing_transaction_failure_message: failure_message {
            __typename
            rich_text_text: text
        }
        routing_transaction_failure_reason: failure_reason
    }
    ... on Withdrawal {
        __typename
        withdrawal_id: id
        withdrawal_created_at: created_at
        withdrawal_updated_at: updated_at
        withdrawal_status: status
        withdrawal_resolved_at: resolved_at
        withdrawal_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        withdrawal_transaction_hash: transaction_hash
        withdrawal_fees: fees {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        withdrawal_block_hash: block_hash
        withdrawal_block_height: block_height
        withdrawal_destination_addresses: destination_addresses
        withdrawal_num_confirmations: num_confirmations
        withdrawal_origin: origin {
            id
        }
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> Transaction:
    # pylint: disable=too-many-return-statements
    if obj["__typename"] == "ChannelClosingTransaction":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.ChannelClosingTransaction import (
            ChannelClosingTransaction,
        )

        return ChannelClosingTransaction(
            requester=requester,
            typename="ChannelClosingTransaction",
            id=obj["channel_closing_transaction_id"],
            created_at=obj["channel_closing_transaction_created_at"],
            updated_at=obj["channel_closing_transaction_updated_at"],
            status=parse_enum(
                TransactionStatus, obj["channel_closing_transaction_status"]
            ),
            resolved_at=obj["channel_closing_transaction_resolved_at"],
            amount=CurrencyAmount_from_json(
                requester, obj["channel_closing_transaction_amount"]
            ),
            transaction_hash=obj["channel_closing_transaction_transaction_hash"],
            fees=CurrencyAmount_from_json(
                requester, obj["channel_closing_transaction_fees"]
            )
            if obj["channel_closing_transaction_fees"]
            else None,
            block_hash=obj["channel_closing_transaction_block_hash"],
            block_height=obj["channel_closing_transaction_block_height"],
            destination_addresses=obj[
                "channel_closing_transaction_destination_addresses"
            ],
            num_confirmations=obj["channel_closing_transaction_num_confirmations"],
            channel_id=obj["channel_closing_transaction_channel"]["id"]
            if obj["channel_closing_transaction_channel"]
            else None,
        )
    if obj["__typename"] == "ChannelOpeningTransaction":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.ChannelOpeningTransaction import (
            ChannelOpeningTransaction,
        )

        return ChannelOpeningTransaction(
            requester=requester,
            typename="ChannelOpeningTransaction",
            id=obj["channel_opening_transaction_id"],
            created_at=obj["channel_opening_transaction_created_at"],
            updated_at=obj["channel_opening_transaction_updated_at"],
            status=parse_enum(
                TransactionStatus, obj["channel_opening_transaction_status"]
            ),
            resolved_at=obj["channel_opening_transaction_resolved_at"],
            amount=CurrencyAmount_from_json(
                requester, obj["channel_opening_transaction_amount"]
            ),
            transaction_hash=obj["channel_opening_transaction_transaction_hash"],
            fees=CurrencyAmount_from_json(
                requester, obj["channel_opening_transaction_fees"]
            )
            if obj["channel_opening_transaction_fees"]
            else None,
            block_hash=obj["channel_opening_transaction_block_hash"],
            block_height=obj["channel_opening_transaction_block_height"],
            destination_addresses=obj[
                "channel_opening_transaction_destination_addresses"
            ],
            num_confirmations=obj["channel_opening_transaction_num_confirmations"],
            channel_id=obj["channel_opening_transaction_channel"]["id"]
            if obj["channel_opening_transaction_channel"]
            else None,
        )
    if obj["__typename"] == "Deposit":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.Deposit import Deposit

        return Deposit(
            requester=requester,
            typename="Deposit",
            id=obj["deposit_id"],
            created_at=obj["deposit_created_at"],
            updated_at=obj["deposit_updated_at"],
            status=parse_enum(TransactionStatus, obj["deposit_status"]),
            resolved_at=obj["deposit_resolved_at"],
            amount=CurrencyAmount_from_json(requester, obj["deposit_amount"]),
            transaction_hash=obj["deposit_transaction_hash"],
            fees=CurrencyAmount_from_json(requester, obj["deposit_fees"])
            if obj["deposit_fees"]
            else None,
            block_hash=obj["deposit_block_hash"],
            block_height=obj["deposit_block_height"],
            destination_addresses=obj["deposit_destination_addresses"],
            num_confirmations=obj["deposit_num_confirmations"],
            destination_id=obj["deposit_destination"]["id"],
        )
    if obj["__typename"] == "IncomingPayment":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.IncomingPayment import IncomingPayment

        return IncomingPayment(
            requester=requester,
            typename="IncomingPayment",
            id=obj["incoming_payment_id"],
            created_at=obj["incoming_payment_created_at"],
            updated_at=obj["incoming_payment_updated_at"],
            status=parse_enum(TransactionStatus, obj["incoming_payment_status"]),
            resolved_at=obj["incoming_payment_resolved_at"],
            amount=CurrencyAmount_from_json(requester, obj["incoming_payment_amount"]),
            transaction_hash=obj["incoming_payment_transaction_hash"],
            origin_id=obj["incoming_payment_origin"]["id"]
            if obj["incoming_payment_origin"]
            else None,
            destination_id=obj["incoming_payment_destination"]["id"],
            payment_request_id=obj["incoming_payment_payment_request"]["id"]
            if obj["incoming_payment_payment_request"]
            else None,
        )
    if obj["__typename"] == "OutgoingPayment":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.OutgoingPayment import OutgoingPayment

        return OutgoingPayment(
            requester=requester,
            typename="OutgoingPayment",
            id=obj["outgoing_payment_id"],
            created_at=obj["outgoing_payment_created_at"],
            updated_at=obj["outgoing_payment_updated_at"],
            status=parse_enum(TransactionStatus, obj["outgoing_payment_status"]),
            resolved_at=obj["outgoing_payment_resolved_at"],
            amount=CurrencyAmount_from_json(requester, obj["outgoing_payment_amount"]),
            transaction_hash=obj["outgoing_payment_transaction_hash"],
            origin_id=obj["outgoing_payment_origin"]["id"],
            destination_id=obj["outgoing_payment_destination"]["id"]
            if obj["outgoing_payment_destination"]
            else None,
            fees=CurrencyAmount_from_json(requester, obj["outgoing_payment_fees"])
            if obj["outgoing_payment_fees"]
            else None,
            payment_request_data=PaymentRequestData_from_json(
                requester, obj["outgoing_payment_payment_request_data"]
            )
            if obj["outgoing_payment_payment_request_data"]
            else None,
            failure_reason=parse_enum_optional(
                PaymentFailureReason, obj["outgoing_payment_failure_reason"]
            ),
            failure_message=RichText_from_json(
                requester, obj["outgoing_payment_failure_message"]
            )
            if obj["outgoing_payment_failure_message"]
            else None,
        )
    if obj["__typename"] == "RoutingTransaction":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.RoutingTransaction import RoutingTransaction

        return RoutingTransaction(
            requester=requester,
            typename="RoutingTransaction",
            id=obj["routing_transaction_id"],
            created_at=obj["routing_transaction_created_at"],
            updated_at=obj["routing_transaction_updated_at"],
            status=parse_enum(TransactionStatus, obj["routing_transaction_status"]),
            resolved_at=obj["routing_transaction_resolved_at"],
            amount=CurrencyAmount_from_json(
                requester, obj["routing_transaction_amount"]
            ),
            transaction_hash=obj["routing_transaction_transaction_hash"],
            incoming_channel_id=obj["routing_transaction_incoming_channel"]["id"]
            if obj["routing_transaction_incoming_channel"]
            else None,
            outgoing_channel_id=obj["routing_transaction_outgoing_channel"]["id"]
            if obj["routing_transaction_outgoing_channel"]
            else None,
            fees=CurrencyAmount_from_json(requester, obj["routing_transaction_fees"])
            if obj["routing_transaction_fees"]
            else None,
            failure_message=RichText_from_json(
                requester, obj["routing_transaction_failure_message"]
            )
            if obj["routing_transaction_failure_message"]
            else None,
            failure_reason=parse_enum_optional(
                RoutingTransactionFailureReason,
                obj["routing_transaction_failure_reason"],
            ),
        )
    if obj["__typename"] == "Withdrawal":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.Withdrawal import Withdrawal

        return Withdrawal(
            requester=requester,
            typename="Withdrawal",
            id=obj["withdrawal_id"],
            created_at=obj["withdrawal_created_at"],
            updated_at=obj["withdrawal_updated_at"],
            status=parse_enum(TransactionStatus, obj["withdrawal_status"]),
            resolved_at=obj["withdrawal_resolved_at"],
            amount=CurrencyAmount_from_json(requester, obj["withdrawal_amount"]),
            transaction_hash=obj["withdrawal_transaction_hash"],
            fees=CurrencyAmount_from_json(requester, obj["withdrawal_fees"])
            if obj["withdrawal_fees"]
            else None,
            block_hash=obj["withdrawal_block_hash"],
            block_height=obj["withdrawal_block_height"],
            destination_addresses=obj["withdrawal_destination_addresses"],
            num_confirmations=obj["withdrawal_num_confirmations"],
            origin_id=obj["withdrawal_origin"]["id"],
        )
    graphql_typename = obj["__typename"]
    raise LightsparkException(
        "UNKNOWN_INTERFACE",
        f"Couldn't find a concrete type for interface Transaction corresponding to the typename={graphql_typename}",
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester


@dataclass
class DeleteApiTokenOutput:
    requester: Requester

    account_id: str


FRAGMENT = """
fragment DeleteApiTokenOutputFragment on DeleteApiTokenOutput {
    __typename
    delete_api_token_output_account: account {
        id
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> DeleteApiTokenOutput:
    return DeleteApiTokenOutput(
        requester=requester,
        account_id=obj["delete_api_token_output_account"]["id"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping, Optional

from lightspark.requests.requester import Requester

from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json


@dataclass
class BlockchainBalance:
    """This object provides a detailed breakdown of a `LightsparkNode`'s current balance on the Bitcoin Network."""

    requester: Requester

    total_balance: Optional[CurrencyAmount]
    """The total wallet balance, including unconfirmed UTXOs."""

    confirmed_balance: Optional[CurrencyAmount]
    """The balance of confirmed UTXOs in the wallet."""

    unconfirmed_balance: Optional[CurrencyAmount]
    """The balance of unconfirmed UTXOs in the wallet."""

    locked_balance: Optional[CurrencyAmount]
    """The balance that's locked by an on-chain transaction."""

    required_reserve: Optional[CurrencyAmount]
    """Funds required to be held in reserve for channel bumping."""

    available_balance: Optional[CurrencyAmount]
    """Funds available for creating channels or withdrawing."""


FRAGMENT = """
fragment BlockchainBalanceFragment on BlockchainBalance {
    __typename
    blockchain_balance_total_balance: total_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    blockchain_balance_confirmed_balance: confirmed_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    blockchain_balance_unconfirmed_balance: unconfirmed_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    blockchain_balance_locked_balance: locked_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    blockchain_balance_required_reserve: required_reserve {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    blockchain_balance_available_balance: available_balance {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> BlockchainBalance:
    return BlockchainBalance(
        requester=requester,
        total_balance=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_total_balance"]
        )
        if obj["blockchain_balance_total_balance"]
        else None,
        confirmed_balance=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_confirmed_balance"]
        )
        if obj["blockchain_balance_confirmed_balance"]
        else None,
        unconfirmed_balance=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_unconfirmed_balance"]
        )
        if obj["blockchain_balance_unconfirmed_balance"]
        else None,
        locked_balance=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_locked_balance"]
        )
        if obj["blockchain_balance_locked_balance"]
        else None,
        required_reserve=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_required_reserve"]
        )
        if obj["blockchain_balance_required_reserve"]
        else None,
        available_balance=CurrencyAmount_from_json(
            requester, obj["blockchain_balance_available_balance"]
        )
        if obj["blockchain_balance_available_balance"]
        else None,
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester


@dataclass
class RichText:
    requester: Requester

    text: str


FRAGMENT = """
fragment RichTextFragment on RichText {
    __typename
    rich_text_text: text
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> RichText:
    return RichText(
        requester=requester,
        text=obj["rich_text_text"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from enum import Enum


class ChannelStatus(Enum):
    ___FUTURE_VALUE___ = "___FUTURE_VALUE___"
    """This is an enum value that represents future values that could be added in the future. Clients should support unknown values as more of them could be added without notice."""
    OK = "OK"
    """The channel is online and ready to send and receive funds."""
    PENDING = "PENDING"
    """The channel has been created, but the Bitcoin transaction that initiates it still needs to be confirmed on the Bitcoin blockchain."""
    OFFLINE = "OFFLINE"
    """The channel is not available, likely because the peer is not online."""
    UNBALANCED_FOR_SEND = "UNBALANCED_FOR_SEND"
    """The channel is behaving properly, but its remote balance is much higher than its local balance so it is not balanced properly for sending funds out."""
    UNBALANCED_FOR_RECEIVE = "UNBALANCED_FOR_RECEIVE"
    """The channel is behaving properly, but its remote balance is much lower than its local balance so it is not balanced properly for receiving funds."""
    CLOSED = "CLOSED"
    """The channel has been closed. Information about the channel is still available for historical purposes but the channel cannot be used anymore."""
    ERROR = "ERROR"
    """Something unexpected happened and we cannot determine the status of this channel. Please try again later or contact the support."""



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

import json
from datetime import datetime, timezone
from enum import Enum


class Encoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Enum):
            return o.name
        if isinstance(o, datetime):
            return o.replace(tzinfo=timezone.utc).isoformat()
        return json.JSONEncoder.default(self, o)



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping, Optional

from lightspark.objects.InvoiceType import InvoiceType
from lightspark.utils.enums import parse_enum_optional

from .InvoiceType import InvoiceType


@dataclass
class CreateInvoiceInput:
    node_id: str

    amount_msats: int

    memo: Optional[str]

    invoice_type: Optional[InvoiceType]


def from_json(obj: Mapping[str, Any]) -> CreateInvoiceInput:
    return CreateInvoiceInput(
        node_id=obj["create_invoice_input_node_id"],
        amount_msats=obj["create_invoice_input_amount_msats"],
        memo=obj["create_invoice_input_memo"],
        invoice_type=parse_enum_optional(
            InvoiceType, obj["create_invoice_input_invoice_type"]
        ),
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester


@dataclass
class CreateNodeWalletAddressOutput:
    requester: Requester

    node_id: str

    wallet_address: str


FRAGMENT = """
fragment CreateNodeWalletAddressOutputFragment on CreateNodeWalletAddressOutput {
    __typename
    create_node_wallet_address_output_node: node {
        id
    }
    create_node_wallet_address_output_wallet_address: wallet_address
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> CreateNodeWalletAddressOutput:
    return CreateNodeWalletAddressOutput(
        requester=requester,
        node_id=obj["create_node_wallet_address_output_node"]["id"],
        wallet_address=obj["create_node_wallet_address_output_wallet_address"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester

from .ApiToken import ApiToken
from .ApiToken import from_json as ApiToken_from_json


@dataclass
class CreateApiTokenOutput:
    requester: Requester

    api_token: ApiToken
    """The API Token that has been created."""

    client_secret: str
    """The secret that should be used to authenticate against our API.
This secret is not stored and will never be available again after this. You must keep this secret secure as it grants access to your account."""


FRAGMENT = """
fragment CreateApiTokenOutputFragment on CreateApiTokenOutput {
    __typename
    create_api_token_output_api_token: api_token {
        __typename
        api_token_id: id
        api_token_created_at: created_at
        api_token_updated_at: updated_at
        api_token_client_id: client_id
        api_token_name: name
        api_token_permissions: permissions
    }
    create_api_token_output_client_secret: client_secret
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> CreateApiTokenOutput:
    return CreateApiTokenOutput(
        requester=requester,
        api_token=ApiToken_from_json(
            requester, obj["create_api_token_output_api_token"]
        ),
        client_secret=obj["create_api_token_output_client_secret"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping, Optional

from lightspark.requests.requester import Requester

from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json


@dataclass
class ChannelFees:
    requester: Requester

    base_fee: Optional[CurrencyAmount]

    fee_rate_per_mil: Optional[int]


FRAGMENT = """
fragment ChannelFeesFragment on ChannelFees {
    __typename
    channel_fees_base_fee: base_fee {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
    channel_fees_fee_rate_per_mil: fee_rate_per_mil
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> ChannelFees:
    return ChannelFees(
        requester=requester,
        base_fee=CurrencyAmount_from_json(requester, obj["channel_fees_base_fee"])
        if obj["channel_fees_base_fee"]
        else None,
        fee_rate_per_mil=obj["channel_fees_fee_rate_per_mil"],
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.exceptions import LightsparkException
from lightspark.objects.BitcoinNetwork import BitcoinNetwork
from lightspark.requests.requester import Requester
from lightspark.utils.enums import parse_enum

from .BitcoinNetwork import BitcoinNetwork
from .CurrencyAmount import from_json as CurrencyAmount_from_json
from .Node import from_json as Node_from_json


@dataclass
class PaymentRequestData:
    """The interface of a payment request on the Lightning Network (a.k.a. Lightning Invoice)."""

    requester: Requester

    encoded_payment_request: str

    bitcoin_network: BitcoinNetwork

    typename: str


FRAGMENT = """
fragment PaymentRequestDataFragment on PaymentRequestData {
    __typename
    ... on InvoiceData {
        __typename
        invoice_data_encoded_payment_request: encoded_payment_request
        invoice_data_bitcoin_network: bitcoin_network
        invoice_data_payment_hash: payment_hash
        invoice_data_amount: amount {
            __typename
            currency_amount_original_value: original_value
            currency_amount_original_unit: original_unit
            currency_amount_preferred_currency_unit: preferred_currency_unit
            currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
            currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
        }
        invoice_data_created_at: created_at
        invoice_data_expires_at: expires_at
        invoice_data_memo: memo
        invoice_data_destination: destination {
            __typename
            ... on GraphNode {
                __typename
                graph_node_id: id
                graph_node_created_at: created_at
                graph_node_updated_at: updated_at
                graph_node_alias: alias
                graph_node_bitcoin_network: bitcoin_network
                graph_node_color: color
                graph_node_conductivity: conductivity
                graph_node_display_name: display_name
                graph_node_public_key: public_key
            }
            ... on LightsparkNode {
                __typename
                lightspark_node_id: id
                lightspark_node_created_at: created_at
                lightspark_node_updated_at: updated_at
                lightspark_node_alias: alias
                lightspark_node_bitcoin_network: bitcoin_network
                lightspark_node_color: color
                lightspark_node_conductivity: conductivity
                lightspark_node_display_name: display_name
                lightspark_node_public_key: public_key
                lightspark_node_account: account {
                    id
                }
                lightspark_node_blockchain_balance: blockchain_balance {
                    __typename
                    blockchain_balance_total_balance: total_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    blockchain_balance_confirmed_balance: confirmed_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    blockchain_balance_unconfirmed_balance: unconfirmed_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    blockchain_balance_locked_balance: locked_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    blockchain_balance_required_reserve: required_reserve {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                    blockchain_balance_available_balance: available_balance {
                        __typename
                        currency_amount_original_value: original_value
                        currency_amount_original_unit: original_unit
                        currency_amount_preferred_currency_unit: preferred_currency_unit
                        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                    }
                }
                lightspark_node_encrypted_signing_private_key: encrypted_signing_private_key {
                    __typename
                    secret_encrypted_value: encrypted_value
                    secret_cipher: cipher
                }
                lightspark_node_total_balance: total_balance {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                lightspark_node_total_local_balance: total_local_balance {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                lightspark_node_local_balance: local_balance {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                lightspark_node_purpose: purpose
                lightspark_node_remote_balance: remote_balance {
                    __typename
                    currency_amount_original_value: original_value
                    currency_amount_original_unit: original_unit
                    currency_amount_preferred_currency_unit: preferred_currency_unit
                    currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
                    currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
                }
                lightspark_node_status: status
            }
        }
    }
}
"""


def from_json(requester: Requester, obj: Mapping[str, Any]) -> PaymentRequestData:
    if obj["__typename"] == "InvoiceData":
        # pylint: disable=import-outside-toplevel
        from lightspark.objects.InvoiceData import InvoiceData

        return InvoiceData(
            requester=requester,
            typename="InvoiceData",
            encoded_payment_request=obj["invoice_data_encoded_payment_request"],
            bitcoin_network=parse_enum(
                BitcoinNetwork, obj["invoice_data_bitcoin_network"]
            ),
            payment_hash=obj["invoice_data_payment_hash"],
            amount=CurrencyAmount_from_json(requester, obj["invoice_data_amount"]),
            created_at=obj["invoice_data_created_at"],
            expires_at=obj["invoice_data_expires_at"],
            memo=obj["invoice_data_memo"],
            destination=Node_from_json(requester, obj["invoice_data_destination"]),
        )
    graphql_typename = obj["__typename"]
    raise LightsparkException(
        "UNKNOWN_INTERFACE",
        f"Couldn't find a concrete type for interface PaymentRequestData corresponding to the typename={graphql_typename}",
    )



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from enum import Enum


class TransactionType(Enum):
    ___FUTURE_VALUE___ = "___FUTURE_VALUE___"
    """This is an enum value that represents future values that could be added in the future. Clients should support unknown values as more of them could be added without notice."""
    OUTGOING_PAYMENT = "OUTGOING_PAYMENT"
    """Transactions initiated from a Lightspark node on Lightning Network."""
    INCOMING_PAYMENT = "INCOMING_PAYMENT"
    """Transactions received by a Lightspark node on Lightning Network."""
    ROUTED = "ROUTED"
    """Transactions that forwarded payments through Lightspark nodes on Lightning Network."""
    L1_WITHDRAW = "L1_WITHDRAW"
    """Transactions on the Bitcoin blockchain to withdraw funds from a Lightspark node to a Bitcoin wallet."""
    L1_DEPOSIT = "L1_DEPOSIT"
    """Transactions on Bitcoin blockchain to fund a Lightspark node's wallet."""
    CHANNEL_OPEN = "CHANNEL_OPEN"
    """Transactions on Bitcoin blockchain to open a channel on Lightning Network funded by the local Lightspark node."""
    CHANNEL_CLOSE = "CHANNEL_CLOSE"
    """Transactions on Bitcoin blockchain to close a channel on Lightning Network where the balances are allocated back to local and remote nodes."""
    PAYMENT = "PAYMENT"
    """Transactions initiated from a Lightspark node on Lightning Network."""
    PAYMENT_REQUEST = "PAYMENT_REQUEST"
    """Payment requests from a Lightspark node on Lightning Network"""
    ROUTE = "ROUTE"
    """Transactions that forwarded payments through Lightspark nodes on Lightning Network."""



# Copyright ©, 2022-present, Lightspark Group, Inc. - All Rights Reserved

from dataclasses import dataclass
from typing import Any, Mapping

from lightspark.requests.requester import Requester

from .CurrencyAmount import CurrencyAmount
from .CurrencyAmount import from_json as CurrencyAmount_from_json


@dataclass
class LightningFeeEstimateOutput:
    requester: Requester

    fee_estimate: CurrencyAmount
    """The estimated fees for the payment."""


FRAGMENT = """
fragment LightningFeeEstimateOutputFragment on LightningFeeEstimateOutput {
    __typename
    lightning_fee_estimate_output_fee_estimate: fee_estimate {
        __typename
        currency_amount_original_value: original_value
        currency_amount_original_unit: original_unit
        currency_amount_preferred_currency_unit: preferred_currency_unit
        currency_amount_preferred_currency_value_rounded: preferred_currency_value_rounded
        currency_amount_preferred_currency_value_approx: preferred_currency_value_approx
    }
}
"""


def from_json(
    requester: Requester, obj: Mapping[str, Any]
) -> LightningFeeEstimateOutput:
    return LightningFeeEstimateOutput(
        requester=requester,
        fee_estimate=CurrencyAmount_from_json(
            requester, obj["lightning_fee_estimate_output_fee_estimate"]
        ),
    )
